@@ -114,8 +114,8 @@ int __connman_agent_report_error(struct connman_service *service,
 #include <connman/log.h>
 
 int __connman_log_init(const char *program, const char *debug,
-						connman_bool_t detach);
-void __connman_log_cleanup(void);
+		connman_bool_t detach, connman_bool_t backtrace);
+void __connman_log_cleanup(connman_bool_t backtrace);
 void __connman_log_enable(struct connman_debug_desc *start,
 					struct connman_debug_desc *stop);
 
@@ -691,7 +691,8 @@ void __connman_service_notify(struct connman_service *service,
 int __connman_service_counter_register(const char *counter);
 void __connman_service_counter_unregister(const char *counter);
 
-struct connman_session;
+#include <connman/session.h>
+
 struct service_entry;
 typedef connman_bool_t (* service_match_cb) (struct connman_session *session,
 					struct connman_service *service);
